package agents;
import java.awt.Color;
import java.util.Arrays;

import ec.util.MersenneTwisterFast;

import sim.engine.SimState;
import sim.engine.Steppable;
import sim.engine.Stoppable;
import sim.field.grid.SparseGrid2D;
import sim.field.network.Edge;
import sim.field.network.Network;
import sim.util.*;

public class Agent implements Steppable{
	public Stoppable event;
	public double endowment;
	public int index;

	public MersenneTwisterFast random;
	public Bag recycledAgents;
	
	public Bag gameBag;
	public int numGamesPlaying = 0;
	public int justJoinedIndex; //index of game just joined this turn. 
	public boolean justJoined; //did agent just join a game this turn?
	public int justLeftIndex;
	public boolean justLeft;
	public int strategy; //Cooperator or Defector
	public double lastPayoff; //total payoff from the most recent round of games.
	
	
	

	public int getNumGames(){return numGamesPlaying;}
	public String getStrategy(){
		String a = (this.strategy == AgentsSimulation.COOPERATOR) ? "C" : "D";
				return a;}
	
	
	/**
	 * Constructor
	 */
	public Agent(double endow, int numGames){
		this.endowment = endow;
		this.gameBag = new Bag();
		//this.playList = new boolean[numGames][2];
	}

	

	
	
	
	/**
	 * How many games is the agent currently involved with? 
	 */
	public int howManyGames(AgentsSimulation as){
		final boolean[] gameMatrix = as.gameMatrix[this.index];
		int count = 0;
		for(int i = 0; i < gameMatrix.length; i++){
			if(gameMatrix[i])
				count++;
		}
		numGamesPlaying = count;
		return count;
	}
	
	
	/**
	 * returns the mean of an array of doubles
	 */
	public double average(double[] vector){
		double sum = 0;
		final double size = (double)vector.length;
		for(int i = 0; i < size; i++)
			sum += vector[i];
		return sum/size;
	}
	
	
	/**
	 * Join a game. 
	 * With some probability, join a random game.
	 * Otherwise: look at all games played by individuals who
	 * are also playing games with you and record the payoffs from those 
	 * games.
	 * With probability proportional to relative payoffs, choose a game to join
	 *
	 */
	public void joinNewGame(AgentsSimulation as){
		if(as.random.nextBoolean(as.joinRandomGame)) //join a random game
			joinRandomGame(as);
		else if (as.joinTheBest) //join strictly the game with biggest payoff
			joinTheBest(as);
		else{
			//Get a Bag of potential games to join. 
			Bag neighboringGames;
			if(as.joinGamesOnlyInNetwork)
				neighboringGames = bagOfNeighboringGames(as);//a bag of the neighbor's games (that I'm not playing)
			else
				neighboringGames = bagOfAllGames(as);
			
			//If there EXIST neighboring games to try and join...
			if(neighboringGames != null && neighboringGames.numObjs > 0){
			//choose a game probabilistically based on relative payouts.
			double[][] nGames = new double[neighboringGames.numObjs][2];
			double sum = 0;
			for(int i = 0; i < neighboringGames.numObjs; i++){
				Game g = (Game)neighboringGames.get(i);
				nGames[i][0] = g.payout;
				nGames[i][1] = g.index;
				sum += g.payout;
			}
			double[] nSum = new double[nGames.length];
			for(int i = 0; i < nGames.length; i++)
				nGames[i][0] = nGames[i][0]/sum;//normalize the payoffs.
			nSum[0] = nGames[0][0];
			for(int i = 1; i < nGames.length; i++)
				nSum[i] = nSum[i-1] + nGames[i][0];
			double rand = as.random.nextDouble();
			
			//basically, goes down the list of games, in whatever order, 
			//and chooses games probabilistically
			/* 4/24/11 -PS
			 * I'm going to change this so that it only picks one game, and tries to join.
			 * If it fails, it can't try again this round. 
			 */
			for(int i = 0; i < neighboringGames.numObjs; i++){
				if(rand < nSum[i]){ 
					//join the game if reputation doesn't matter, or if the members vote me in.
					if(!as.useReputation || reputationVote((int)nGames[i][1], as))
						joinThisGame((int)nGames[i][1], as);
					break;
				}
			}
			
			
		}
		}
	}
	
	
	/**Join a random game.*/
	public void joinRandomGame(AgentsSimulation as){
		Bag games = as.gamesSpace.getAllObjects();
		int i = as.random.nextInt(as.numGames);
		final boolean[][] gameMatrix = as.gameMatrix;
		Game g = (Game)games.get(i);
		int count = 0;
		while(count < as.numGames && (gameMatrix[this.index][i] || g.howManyPlayers(as) >= as.maxPlayers)){
			i = as.random.nextInt(as.numGames);
			g = (Game)games.get(i);
			count++;
		}
		if(g.howManyPlayers(as) < as.maxPlayers && (!as.useReputation || reputationVote(i, as)))
			joinThisGame(i, as);
	}
	
	
	
	/**
	 * Join the game with the highest payoff
	 */
	public void joinTheBest(AgentsSimulation as){
		Bag neighboringGames = bagOfNeighboringGames(as);//a bag of the neighbor's games (that I'm not playing)
		if(neighboringGames != null && neighboringGames.numObjs > 0){
			Double[][] nGames = new Double[neighboringGames.numObjs][2];
			for(int i = 0; i < neighboringGames.numObjs; i++){
				Game g = (Game)neighboringGames.get(i);
				nGames[i][0] = g.payout;
				nGames[i][1] = (double)g.index;
			}
			Arrays.sort(nGames, new TwoDComparatorDblDescend()); //sort by payoffs
			int index = (int)Math.round(nGames[0][1]);
			joinThisGame(index, as);
		}
	}
	
	/**
	 * Agent joins the game with the given index
	 */
	public void joinThisGame(int index, AgentsSimulation as){
		Bag games = as.gamesSpace.getAllObjects();
		Game g = (Game)games.get(index);
		if(as.gameMatrix[this.index][g.index]){
			System.out.println("ERROR: Already a member of game trying to join.");
			//event.stop();
		}
		as.gameMatrix[this.index][g.index] = true;
		justJoinedIndex = index;
		//System.out.println("Edge added from Agent " + this.index + " to Game " + g.index);
	}
	
	/**
	 * Returns a bag of all the agents who are playing at least one game
	 * in common with this agent.
	 */
	public Bag bagOfNeighbors(AgentsSimulation as){
		final boolean[][] gameMatrix = as.gameMatrix;
		Bag neighbors = new Bag();
		Bag agents = as.agentsSpace.getAllObjects();
		for(int j = 0; j < as.numGames; j++){
			if(gameMatrix[this.index][j]) //if this agent is playing game j
				for(int i = 0; i < as.numAgents; i++)
					if(gameMatrix[i][j] && i != this.index){
						Agent a = (Agent)agents.get(i);
						if(!neighbors.contains(a))//only add it once
							neighbors.add(a);
					}
		}
		return neighbors;
	}
	
	/**
	 * Returns a bag of all the games not currently being played but being
	 * played by at least one agent that shares a game.
	 * ERROR.
	 */
	public Bag bagOfNeighboringGames(AgentsSimulation as){
		Bag neighboringGames = new Bag();
		Bag games = as.gamesSpace.allObjects;
		IntBag coplayerGames = this.coplayerGames(as); //indices of separate co-player games.
		for(int i = 0; i < coplayerGames.numObjs; i++){
			int x = coplayerGames.get(i);
			Game g = (Game)games.get(x);
			if(!neighboringGames.contains(g) && g.howManyPlayers(as) < as.maxPlayers)
				neighboringGames.add(g);
		}
		return neighboringGames;
	}
	
	/**
	 * Returns a bag of all the games not currently being played
	 */
	public Bag bagOfAllGames(AgentsSimulation as){
		Bag neighboringGames = new Bag();
		Bag neighbors = bagOfNeighbors(as);
		Bag myGames = this.currentGames(as);
		if(neighbors != null){//as long as it has some neighbors
			for(int i = 0; i < neighbors.numObjs; i++){//go through each neigbor
				Agent a = (Agent)neighbors.get(i);
				Bag agames = a.currentGames(as);//bag of that agent's games
				for(int j = 0; j < agames.numObjs; j++){//add the neighbor's nonoverlapping games.
					Game g = (Game)agames.get(j);
					if(!neighboringGames.contains(g) && !myGames.contains(g) && g.howManyPlayers(as) < as.maxPlayers)
						neighboringGames.add(g);
				}	
			}
		}
		return neighboringGames;
	}
	
	/**
	 * Returns a bag of the current games an agent is in.
	 */
	public Bag currentGames(AgentsSimulation as){
		Bag cg = new Bag();
		Bag games = as.gamesSpace.getAllObjects();
		final boolean[][] gameMatrix = as.gameMatrix;
		for(int i = 0; i < as.numGames; i++)
			if(gameMatrix[this.index][i]){
				Game g = (Game)games.get(i);
				cg.add(g);
			}	
		return cg;
	}
	
	/**
	 * Returns an IntBag of the indices of the agent's current games.
	 */
	public IntBag currentGamesInd(AgentsSimulation as){
		IntBag cg = new IntBag();
		final boolean[][] gameMatrix = as.gameMatrix;
		for(int i = 0; i < as.numGames; i++)
			if(gameMatrix[this.index][i]){
				cg.add(i);
			}	
		return cg;
	}
	
	/**
	 * returns the current games for agent with the given index.
	 */
	public IntBag currentGamesInd(int index, AgentsSimulation as){
		IntBag cg = new IntBag();
		final boolean[][] gameMatrix = as.gameMatrix;
		for(int i = 0; i < as.numGames; i++)
			if(gameMatrix[index][i]){
				cg.add(i);
			}	
		return cg;
	}
	
	
	/**
	 * return the indices of all games being played by current co-players that agent
	 * is NOT currently playing.
	 */
	public IntBag coplayerGames(AgentsSimulation as){
		IntBag coplayerGames = new IntBag();
		IntBag currGames = this.currentGamesInd(as); //this agent's current games.
		IntBag coplayers = this.coplayers(as); //this agent's coplayers
		for(int i = 0; i < coplayers.numObjs; i++){//go through each coplayer
			int a = coplayers.get(i); //coplayer index.
			IntBag cg = currentGamesInd(a, as); //coplayer's current games.
			for(int j = 0; j < cg.numObjs; j++){//go through coplayer's current games.
				int g = cg.get(j);
				if(!currGames.contains(g) && !coplayerGames.contains(g)) // add if our agent isn't already playing this game.
					coplayerGames.add(g);
			}
		}
		return coplayerGames;
	}
	
	/**
	 * returns the indices of all the current co-players (agents that share at least one game).
	 */
	public IntBag coplayers(AgentsSimulation as){
		IntBag coplayers = new IntBag();
		final int numAgents = as.numAgents;
		IntBag currentGames = currentGamesInd(as);//all the indices of the agent's current games.
		for(int i = 0; i < numAgents; i++){ //check all agents
			if(i != this.index){ //skip itself
				IntBag cg = currentGamesInd(i, as);
				for(int j = 0; j < cg.numObjs; j++){
					int x = cg.get(j);
					if(currentGames.contains(x)){ //if this agent is also playing one of i's games, add i.
						coplayers.add(i);
						break;
					}
				}
			}
		}
		return coplayers;
	}
	
	
	/**
	 * Checks the agents within-game reputation to the mean, and only lets it join if it's good enough.
	 * NOT USED IN THIS VERSION.
	 */
	public boolean goodEnough(int index, AgentsSimulation as){
		Bag games = as.gamesSpace.getAllObjects();
		Game g = (Game)games.get(index);
		if(!as.useReputation)
			return true;
		else if (g.gameReputation(this, as) >= g.meanGameReputation(as))
			return true;
		else return false; 
		
	}
	
	
	
	
	/**
	 * The current players of a game vote on whether to allow this 
	 * new player to join, based on his reputation with all of them, and a threshold for voting.
	 */
	public boolean reputationVote(int index, AgentsSimulation as){
		Bag games = as.gamesSpace.getAllObjects();
		Game g = (Game)games.get(index);
		IntBag players = g.currentPlayers(as);
		final double numPlayers = players.numObjs;
		double yesVotes = 0;
		for(int i = 0; i < numPlayers; i++){
			int ind = players.get(i);
			double imageScore = as.reputation[ind][this.index];
			double prob = as.logistic(imageScore);
			if(as.random.nextBoolean(prob))
				yesVotes++;
		}
		
		if(yesVotes/numPlayers > as.reputationThreshold)
			return true;
		else 
			return false;
	}

	
	/**
	 * Leave a game. 
	 * Look at all games currently playing
	 * Unlike joining games, which involves more uncertainty, 
	 * We will simply leave the game with the lowest payoff.
	 * However, we can leave a nonzero probability of leaving a random game.
	 */
	public void leaveNewGame(AgentsSimulation as){
		if(as.random.nextBoolean(as.leaveRandomGame))
			leaveRandomGame(as);
		else{
			Bag cg = currentGames(as); //bag of current games.
			if(justJoined)//remove game just joined, if any
				for(int i = 0; i < cg.numObjs; i++){
					Game g = (Game)cg.get(i);
					if(g.index == justJoinedIndex){
						cg.remove(g);
						break;
					}
				}
			//choose game with the lowest payoff and leave.
			Double[][] payoffs = new Double[cg.numObjs][2];
			for(int i = 0; i < cg.numObjs; i++){
				Game g = (Game)cg.get(i);
				payoffs[i][0] = g.payout;
				payoffs[i][1] = (double)g.index;
			}
			Arrays.sort(payoffs, new TwoDComparatorDblAscend()); //sort by payoffs
			int index = (int)Math.round(payoffs[0][1]);
			double gp = payoffs[0][0];
			double d = (as.distributedContribution) ? this.howManyGames(as) : 1.0;
			if(!as.leaveOnlyLosingGames || 
					(this.strategy == AgentsSimulation.COOPERATOR && gp <= (as.contribution/d)) ||
					(this.strategy == AgentsSimulation.DEFECTOR && gp <= 0)
					) //possibly only leave if it's a losing game.
				leaveThisGame(index, as);
			}
	}
	
	/**Leave a random game.*/
	public void leaveRandomGame(AgentsSimulation as){
		Bag cg = currentGames(as); //bag of current games.
		if(justJoined)//remove game just joined, if any
			for(int i = 0; i < cg.numObjs; i++){
				Game g = (Game)cg.get(i);
				if(g.index == justJoinedIndex){
					cg.remove(g);
					break;
				}
			}
		int x = as.random.nextInt(cg.numObjs);
		Game g = (Game)cg.get(x);
		leaveThisGame(g.index, as);
	}
	
	
	
	/**
	 * Agent leaves the game with the given index
	 */
	public void leaveThisGame(int index, AgentsSimulation as){
		justLeftIndex = index;
		Bag games = as.gamesSpace.getAllObjects();
		Game g = (Game)games.get(index);
		if(!as.gameMatrix[this.index][g.index]){
			System.out.println("ERROR: Not a member of game trying to leave.");
			//event.stop();
		}
		as.gameMatrix[this.index][g.index] = false;
		
	}
	
	
	/**
	 * returns true if agent is a player in game g, false otherwise.
	 */
	public boolean isAMember(Game g, AgentsSimulation as){
		if(as.gameMatrix[this.index][g.index])
			return true;
		return false;
	}
	
	/**
	 * returns true if agent is a player in the game with index i, false otherwise.
	 */
	public boolean isAMember(int i, AgentsSimulation as){
		if(as.gameMatrix[this.index][i])
			return true;
		return false;
	}
	

	/*
	 * Sets a portrayal for the agent. This will determine the shape and color of the agent 
	 * in the graphical display.
	 * NOT CURRENTLY USED
	 */
	public void setPortrayal(AgentsSimulation as, Agent a){          
		as.agentsPortrayal.setPortrayalForObject(a,new sim.portrayal.simple.OvalPortrayal2D(new Color((float)(1.0),(float)0.0,(float)0.0)));
	}
	
	
	/**
	 * Returns the assortment index for the agent. 
	 */
	public double assortmentIndex(AgentsSimulation as){
		int numGames = this.howManyGames(as); //number of games.
		Bag games = currentGames(as); //bag of the agent's current games
		double assortment = 0;
		
		for (int i = 0; i < numGames; i++){ //for each game
			Game g = (Game)games.get(i);
			int n = g.howManyPlayers(as);
			int c = g.howManyCooperators(as);
			int nLike = (this.strategy == AgentsSimulation.COOPERATOR) ? c : n - c;
			assortment += (double)nLike/(double)(n);
		}
		
		return assortment/(double)numGames;
		
	}
	
	
	
	


	/*********************************************************************************************************
	 * 					Step Method
	 ***********************************************************************************************************/
	public void step(SimState state){
		final AgentsSimulation as = (AgentsSimulation)state;
		
		//JOIN
		if(as.random.nextBoolean(as.probJoin)&& this.howManyGames(as) < as.maxGames && this.howManyGames(as) < as.numGames){//join games
			joinNewGame(as);
			justJoined = true;
		}
		else justJoined = false;
		
		//LEAVE
		if(this.howManyGames(as) > as.maxGames || (as.random.nextBoolean(as.probLeave) && this.howManyGames(as) > 1)){//leave games
			leaveNewGame(as);
			justLeft = true;
		}
		else justLeft = false;

	}//end step method


}//end class
