package agents; 

import sim.engine.*;
import sim.util.*;


public class Data {

	/*The String array fileheaders should have all the variables to be collected, in the order 
	 * They are added to the Bag data in the getData method. */
	public String[] fileheaders ={"run", "sim", "time", "activeGames", 
			"avgGamesC", "avgGamesD","avgGamesALL", "SDCoop", "SDDefect", "SDAll",  //games per agent
			"payoffC",  "payoffD","pcSD", "pdSD", "RCP",									//payoffs
			"coops/Game", "defects/Game", "agents/Game", "cpg-SD", "dpg-SD", "apgSD", //agents per game
			"coopFreq/Game", "cfpgSD", "defectFreq/Game", "dfpgSD",			//C and D frequencies in games
			"assort", "timeRCPover1"};			//excess assortment									 
	
				
	public Bag data = null; 
	public int runCount = 0;
	public int simCount = 0;


	/**
	 * Returns a Bag with all the values desired by the user for a data file.
	 */
	public  Bag getData(final SimState state){
		final AgentsSimulation as = (AgentsSimulation)state;  // gets the state of the simulation
		data = new Bag(); 
		//run, sim
		data.add((double)(runCount+1));
		data.add((double)simCount);
		data.add((double)((AgentsSimulation)state).schedule.getSteps());//Time of the data collection
		data.add((double)as.activeGames()); //number of active games
		data.add(average(as.coopConnectivity()));//average games per coop
		data.add(average(as.defectConnectivity()));//average games per defector
		data.add(average(as.agentConnectivity()));//average games per player
		data.add(stdev(as.coopConnectivity()));
		data.add(stdev(as.defectConnectivity()));
		data.add(stdev(as.agentConnectivity()));
		double ac = average(as.coopPayoffs);
		double ad = average(as.defectPayoffs);
		
		data.add(ac);//avg coop payoff
		data.add(ad);//ave defector payoff
		data.add(stdev(as.coopPayoffs));
		data.add(stdev(as.defectPayoffs));
		data.add(ac/ad); //RCP
		
		data.add(average(as.coopGameSizesExclusive()));//avg coops per game
		data.add(average(as.defectGameSizesExclusive()));//avg defectors per game
		data.add(average(as.gameSizesExclusive()));//avg players per game
		data.add(stdev(as.coopGameSizesExclusive()));
		data.add(stdev(as.defectGameSizesExclusive()));
		data.add(stdev(as.gameSizesExclusive()));
		
		data.add(average(as.gameCoopFreqs()));//avg cFreq per game.
		data.add(stdev(as.gameCoopFreqs()));
		
		data.add(average(as.gameDefectorFreqs()));//avg dFreq per game.
		data.add(stdev(as.gameDefectorFreqs()));
		
		data.add(as.calculateAssortment()); //excess assortment.
		data.add(as.timePastRCP); //the time RCP passes one. return zero if never.
		
		return data;
	}

	
	/**
	 * returns the mean of an array of doubles
	 */
	public double average(double[] vector){
		double sum = 0;
		final double size = (double)vector.length;
		for(int i = 0; i < size; i++)
			sum += vector[i];
		return sum/size;
	}
	
	/**
     * calculates the standard deviation of an array of double.
     */
    public double stdev(double[] vector){
      double avg = average(vector);
  	  double val = 0.0;
  	  double sumSq = 0.0;
  	  for(int i = 0; i < vector.length; i++)
  		  sumSq += Math.pow((double)(vector[i] - avg), 2.0); //sum of the squares.
  	  val = (1/(double)(vector.length - 1))*sumSq;
  	  val = Math.sqrt(val);
  	  return val;
    }
	
}//end class







