package agents;

import sim.field.network.Edge;
import sim.util.Bag;
import sim.util.IntBag;

/*
 * This class will represent the public goods game that an actor can belong to. 
 */

public class Game {
	public double production;
	public double payout = 0; //the return from this game on a given round
	public int index;
	AgentsSimulation as;
	
	
	public int getIndex(){return index;}
	public int getNumPlayers(){return howManyPlayers(as);}
	public int getNumCooperators(){ return howManyCooperators(as);}
	public double getproduction(){return production;}
	public double getpayout(){return payout;}
	//get average payoff
	
	public Game(double p, AgentsSimulation as){
		this.production = p;
		this.as = as;
	}
	
	public int howManyPlayers(AgentsSimulation as){
		final boolean[][] gameMatrix = as.gameMatrix;
		int count = 0;
		for(int i = 0; i < as.numAgents; i++)
			if(gameMatrix[i][this.index])
				count++;
		return count;
	}
	
	public int howManyCooperators(AgentsSimulation as){
		Bag agents = as.agentsSpace.getAllObjects();
		final boolean[][] gameMatrix = as.gameMatrix;
		int count = 0;
		for(int i = 0; i < as.numAgents; i++){
			if(gameMatrix[i][this.index]){
				Agent a = (Agent)agents.get(i);
				if(a.strategy == AgentsSimulation.COOPERATOR)
					count++;
			}
		}
		return count;	
	}
	
	/**
	 * returns an array of the indices of all the current players
	 */
	public IntBag currentPlayers(AgentsSimulation as){
		final boolean[][] gameMatrix = as.gameMatrix;
		IntBag players = new IntBag();
		for(int i = 0; i < as.numAgents; i++)
			if(gameMatrix[i][this.index])
				players.add(i);
		return players;
	}
	
	
	/**
	 * Is the agent with the given index playing this game?
	 */
	public boolean isPlaying(int index, AgentsSimulation as){
		final boolean[][] gameMatrix = as.gameMatrix;
		if(gameMatrix[index][this.index])
			return true;
		else return false;
	}
	
	
	
	
	
	/**
	 * Is the agent allowed to join the game? 
	 * I think we should just do it like this: 
	 * the agent is allowed to join with probability equal to its gameReputation, which varies between zero and 1. 
	 */
	public boolean allowJoin(Agent a, AgentsSimulation as){
		if(as.random.nextBoolean(gameReputation(a, as)))
			return true;
		else return false;
	}
	
	
	
	
	
	
	
	
	
	/**
	 * What is the average within-game reputation of an agent. 
	 */
	public double gameReputation(Agent a, AgentsSimulation as){
		IntBag players = this.currentPlayers(as);
		//Bag agents = as.agentsSpace.getAllObjects();
		final double [][] reputation = as.reputation;
		final int selfIndex = a.index;
		double num = 0;
		double denom = 0;
		for(int i = 0; i < players.numObjs; i++){
			int index = players.get(i);
			if(index != selfIndex){
				num += logistic(reputation[i][selfIndex]);
				denom++;
			}
		}
		double r = (denom > 0) ? num/denom : 0.5;
		return r;
		}

	
	/*
	 * What is the mean within-game reputation of ALL the players?
	 */
	public double meanGameReputation(AgentsSimulation as){
		
		IntBag players = this.currentPlayers(as);
		final int numPlayers = players.numObjs;
		Bag agents = as.agentsSpace.getAllObjects();
		double numerator = 0;
		for(int i = 0; i < numPlayers; i++){
			Agent a = (Agent)agents.get(players.get(i));
			numerator += gameReputation(a, as);
		}

		double r = (numPlayers > 0) ? numerator/numPlayers : 0.5;
		return r;
	}

	/**
	 * Returns the logistic function of x, with discounting factor delta.
	 */
	public double logistic(double x, double delta){
		double p = 1 + Math.exp((-1)*delta*x);
		return (1/p);
	}
	
	public double logistic(double x){
		return logistic(x, 1);
	}
	
	
}//end class
