package agents;

import java.awt.Color;

import sim.engine.SimState;
import sim.engine.Steppable;
import sim.engine.Stoppable;
import sim.field.network.Edge;
//import sim.field.network.Network;
import sim.portrayal.Portrayal;
import sim.util.Bag;
import sim.util.IntBag;

public class GameEcology implements Steppable{

	Portrayal gamePortrayal = new sim.portrayal.simple.RectanglePortrayal2D(new Color((float)0.1, (float)0.7, (float)0.2, 1), 2);
    Portrayal gamePortrayalDead = new sim.portrayal.simple.RectanglePortrayal2D(new Color((float)0.3, (float)0.4, (float)0.2, (float)0.2), 2);
    
    public Stoppable event;
	Bag games;
	public GameEcology(){
		
	}


	
	/******************************* STEP METHOD *****************************************/
	public synchronized void  step(SimState state) {
		AgentsSimulation as = (AgentsSimulation)state;
		games = as.gamesSpace.getAllObjects();
		if(as.doGraphics)
			updateNetwork(as);//update all the edges for the display.
		playGames(as);
		updateAgentPayoffs(as);
		//updateAssessments(as);
		if(as.useReputation)
			updateAssessments2(as); //This method is faster, but only counts ignores the influence of multiple shared games.
}// **************************end step****************************************************
	
	
	public void updateAgentPayoffs(AgentsSimulation as){
		final Bag agents = as.agentsSpace.getAllObjects();
		final Bag games = as.gamesSpace.getAllObjects();
		final int numAgents = as.numAgents;
		final int numCoop = (int)((double)numAgents * as.coopFreq);
		final int numDefect = numAgents - numCoop;
		final boolean[][] gameMatrix = as.gameMatrix;
		as.coopPayoffs = new double[numCoop];
		as.defectPayoffs = new double[numDefect];
		int c = 0, d = 0;
		for(int i = 0; i < numAgents ; i++){
			Agent a = (Agent)agents.get(i);
			double payoff = 0;
			for(int j = 0; j < as.numGames; j++){
				if(gameMatrix[i][j]){
					Game g = (Game)games.get(j);
					payoff += g.payout;
				}
			}
			if(a.strategy == AgentsSimulation.DEFECTOR){
				as.defectPayoffs[d] = payoff;
				d++;
			}
			else{//if a cooperator
				if(as.distributedContribution)
					as.coopPayoffs[c] = payoff - as.contribution;
				else
					as.coopPayoffs[c] = payoff - (as.contribution * a.howManyGames(as));
				c++;
			}
		}	
	}
	
	public void playGames(AgentsSimulation as){
		Bag agents = as.agentsSpace.getAllObjects();
		final double contribution = as.contribution;
		final boolean distributedContribution = as.distributedContribution;
		//go through all the games and run each one.
		for(int i = 0; i < games.numObjs; i++){
			Game g = (Game)games.get(i);
			
			//how much does each game take in?
			double intake = 0;
			for(int j = 0; j < as.numAgents; j++){
				if(as.gameMatrix[j][g.index]){
					Agent a = (Agent)agents.get(j);
					if(a.strategy == AgentsSimulation.COOPERATOR){	
						double d = (distributedContribution) ? a.howManyGames(as) : 1.0;
						intake += contribution/d;
					}
				}
			}
			 g.payout = (intake * g.production)/(double)g.howManyPlayers(as);
			final int plrs = g.howManyPlayers(as);
			 
			if(!as.fancyColors){
				if(plrs < 1){
					 g.payout = 0;
					 as.agentsPortrayal.setPortrayalForObject(g, gamePortrayalDead);
				 }
				 else if(plrs == 1)
					 g.payout = intake;
				 if(plrs >= 1)
					 as.agentsPortrayal.setPortrayalForObject(g, gamePortrayal);
				 
			}
			else{
			 if(plrs < 1){
				 g.payout = 0;
				 as.agentsPortrayal.setPortrayalForObject(g, gamePortrayalDead);
			 }
			 else if(plrs == 1)
				 g.payout = intake;
			 if(plrs >= 1){
				 float green = (float)0.7;
				 float red = (float)0.5;
				 float blue = (float)0.5;
				 double coopFreq = (double)g.howManyCooperators(as)/(double)plrs;
				 if(coopFreq > .99){
					 blue = (float)1.0;
					 red = (float)0.0; 
					 green = (float)0.3;
				 }
				 else if(coopFreq < .01){
					 red = (float)1.0;
					 blue = (float)0.0;
					 green = (float)0.1;
				 }
				 else if(coopFreq > 0.501){
					 blue = (float)(coopFreq*0.9 + 0.1);
					 red = (float)0.2;
					 green = (float)0.7;
				 }
				 else if(coopFreq < 0.499){
					 red = (float)((1 - coopFreq)*0.5 + 0.4);
					 blue = (float)0.2;
					 green = (float)0.3;
				 }
				 else {
					 red = (float)0.7;
					 blue = (float)0.7;
					 green = (float)0.4; 
				 }

				 Portrayal gamePortrayal = new sim.portrayal.simple.RectanglePortrayal2D(new Color(red, green, blue, 1), 2);
				 as.agentsPortrayal.setPortrayalForObject(g, gamePortrayal);
				 
			 }
			}
		}
	}
	
	
	/**
	 * Update the reputation matrix. 
	 * The way we have it outlined, there's just no more efficient way to do this that I can think of. 
	 * We have to: 
	 * - Go through each agent. 
	 * - Go through each of its games. 
	 * - Check each co-player and update its reputation. 
	 * 
	 * Ugh.
	 */
	public void updateAssessments(AgentsSimulation as){
		Bag agents = as.agentsSpace.getAllObjects();
		for(int i = 0; i < as.numAgents; i++){ //go through each agent
			Agent a = (Agent)agents.get(i);
			Bag gam = a.currentGames(as);
			for(int j = 0; j < gam.numObjs; j++){ //go through each of its games
				Game g = (Game)gam.get(j);
				IntBag players = g.currentPlayers(as);
				for(int k = 0; k < players.numObjs; k++){ //check each player
					int p = players.get(k);
					if(p != a.index){
						Agent b = (Agent)agents.get(p);
						as.reputation[i][p] += (int)(2*b.strategy - 1); //works because coop = 1, defect = 0;
					}
				}
			}
		}
	}
	
	/**
	 * Trying this a more efficient way.
	 * An important question is whether the number of games that two players are in together 
	 * should matter, in terms of reputation. 
	 * the original updateAssessments method (above) counts every incidence of co-playing
	 * as separate, and so changes it for each game. 
	 * This version will make it so we just do it once. We should probably compare the two, and see if
	 * it matters. 
	 * 
	 * Importantly, this version also allows out of sight, out of mind. 
	 */
	public void updateAssessments2(AgentsSimulation as){
		//TODO allow multiple instances to matter.
		as.updateCoplayerMatrix(); //need to update the co-player matrix.
		final boolean[][] coPlayers = as.coPlayerMatrix;
		Bag agents = as.agentsSpace.getAllObjects();
		final boolean[][] gameMatrix = as.gameMatrix;
		
		
		int x = 1;
		for(int i = 0; i < as.numAgents; i++){
			//Agent a = (Agent)agents.get(i);
			for(int j = 0; j < as.numAgents; j++){
				if(coPlayers[i][j]){
					Agent b = (Agent)agents.get(j);
					if(as.multipleObservances)
						x = numSharedGames(i, j, as);
					as.reputation[i][j] += x * (int)(2*b.strategy - 1); //works because coop = 1, defect = 0
				}
				//Out of sight, out of mind:
				else if(as.outOfSightOutOfMind && as.reputation[i][j] > 0.5)
					as.reputation[i][j] -= 1;
				else if(as.outOfSightOutOfMind && as.reputation[i][j] < -0.5)
					as.reputation[i][j] += 1;
			}
			
		}
	}
		
	
	/**
	 * returns the number of games that currently include both the players indexed by a and b.
	 */
	public int numSharedGames(int a, int b, AgentsSimulation as){
		final boolean[][] gameMatrix = as.gameMatrix;
		boolean [] gamesA = gameMatrix[a];
		boolean [] gamesB = gameMatrix[b];
		int numGames = 0;
		for(int i = 0; i < gamesA.length; i++){
			if(gamesA[i] && gamesB[i])
				numGames++;
		}
		return numGames;
	}
	
	public void printMatrix(AgentsSimulation as){
		final boolean[][] gameMatrix = as.gameMatrix;
		System.out.print("Matrix" + "\t");
		for(int i = 0; i < as.numGames; i++)
			System.out.print(i + "\t");
		System.out.println("");
		for(int i = 0; i < as.numAgents; i++){
			System.out.print(i + "\t");
			for(int j = 0; j < as.numGames; j++){
				System.out.print(gameMatrix[i][j] + "\t");
			}
			System.out.println("");
		}
	}
	
	
	public void updateNetwork(AgentsSimulation as){
		Bag games = as.gamesSpace.getAllObjects();
		Bag agents = as.agentsSpace.getAllObjects();
		//first, remove all the edges.
		for(int i = 0; i < games.numObjs; i++){
			Game g = (Game)games.get(i);
			Bag edges = as.links.getEdgesIn(g);
			if(edges != null && edges.numObjs > 0)
				for(int j = 0; j < edges.numObjs; j++){
					Edge e = (Edge)edges.get(j);
					as.links.removeEdge(e);
				}	
		}
		//next, add all the edges.
		for(int i = 0; i < agents.numObjs; i++)
			for(int j = 0; j < games.numObjs; j++){
				if(as.gameMatrix[i][j]){
					Agent a = (Agent)agents.get(i);
					Game g = (Game)games.get(j);
					as.links.addEdge(a, g, 1);
				}
			}
	}
	
	
	
	
}//end class
