package agents;


import sim.engine.*;
import sim.field.grid.SparseGrid2D;
import sim.display.*;
import sim.portrayal.Portrayal;
import sim.portrayal.grid.*;
import sim.portrayal.inspector.PropertyInspector;
import sim.portrayal.network.NetworkPortrayal2D;
import sim.portrayal.network.SimpleEdgePortrayal2D;
import sim.portrayal.network.SpatialNetwork2D;
/*
import org.jfree.*;
import org.jfree.chart.plot.XYPlot;
import org.jfree.data.general.SeriesChangeEvent;
import org.jfree.data.general.SeriesChangeListener;
*/
import java.awt.*;

import javax.swing.*;

import agents.states.SimStateWithSimController;
import sim.util.*;

public class GraphicalUI extends GUIState{
	public Display2D display; 
    public JFrame displayFrame;
    public Display2D display2; 
    public JFrame displayFrame2;
	static SimStateWithSimController ssc;
	NetworkPortrayal2D edgePortrayal = new NetworkPortrayal2D();
    SparseGridPortrayal2D nodePortrayal = new SparseGridPortrayal2D();
    Portrayal coopPortrayal = new sim.portrayal.simple.OvalPortrayal2D(Color.blue);
    Portrayal defectPortrayal = new sim.portrayal.simple.OvalPortrayal2D(Color.red);
    Portrayal gamePortrayal = new sim.portrayal.simple.RectanglePortrayal2D(new Color((float)0.1, (float)0.7, (float)0.2, 1), 2);
    Portrayal gamePortrayalDead = new sim.portrayal.simple.RectanglePortrayal2D(new Color((float)0.3, (float)0.4, (float)0.2, (float)0.2), 2);
    
    FastValueGridPortrayal2D ecologyPortrayal = new FastValueGridPortrayal2D("Ecology");
    
    
    public void setupPortrayals() {
    	AgentsSimulation as = (AgentsSimulation)state;
    	SparseGrid2D allSpace = new SparseGrid2D(as.gridWidth, as.gridHeight); //as.agentsSpace;
    	Bag games = as.gamesSpace.getAllObjects();
    	for(int i=0; i < games.numObjs; i++){
    		Game g = (Game)games.get(i);
    		Int2D gLoc = as.gamesSpace.getObjectLocation(g);
    		allSpace.setObjectLocation(g, gLoc);
    	}
    	Bag agts = as.agentsSpace.getAllObjects();
    	for(int i=0; i < agts.numObjs; i++){
    		Agent g = (Agent)agts.get(i);
    		Int2D gLoc = as.agentsSpace.getObjectLocation(g);
    		allSpace.setObjectLocation(g, gLoc);
    	}
    	
    	if(as.doGraphics){
    		final Color[] cm = {Color.black, Color.blue, Color.red};
    		ecologyPortrayal.setMap(new sim.util.gui.SimpleColorMap(cm));
    		ecologyPortrayal.setField(as.ecologyGrid);
    		
    		
    		edgePortrayal.setField( new SpatialNetwork2D( allSpace, as.links ) );
            edgePortrayal.setPortrayalForAll(new SimpleEdgePortrayal2D());
            nodePortrayal.setField( allSpace );
            
    		
    		
    		Bag bag = allSpace.getAllObjects();
    		Bag agents = as.agentsSpace.getAllObjects();
    		Agent a = (Agent)agents.objs[0];
    		for(int i=0;i<bag.numObjs;i++){
    			Object p = bag.objs[i]; 
    			if(p.getClass().isInstance(a)){
    				Agent ap = (Agent)p;
    				if(ap.strategy == AgentsSimulation.COOPERATOR)
    					nodePortrayal.setPortrayalForObject(p,coopPortrayal); 
    				else nodePortrayal.setPortrayalForObject(p,defectPortrayal); 
    			}
    			else{
    				Game gp = (Game)p;
    				if(gp.howManyPlayers(as) > 0)
    					nodePortrayal.setPortrayalForObject(p,gamePortrayal);
    				else{
    					nodePortrayal.setPortrayalForObject(p,gamePortrayalDead);
    					System.out.println("Dead game");
    				}
    			}
    		}

    		as.agentsPortrayal = nodePortrayal;  
    		display.reset(); 
    		display.repaint(); // call the repaint method
    		display2.reset(); 
    		display2.repaint(); // call the repaint method
    	}
}
    
    public void init(Controller c){
    	AgentsSimulation as = (AgentsSimulation)state;
        super.init(c);  
        display = new Display2D(600,600,this,1); 
        displayFrame = display.createFrame(); 
        c.registerFrame(displayFrame);   
        displayFrame.setVisible(true);  
        display.setBackdrop(Color.white);
        
        display2 = new Display2D(600,200,this,1); 
        displayFrame2 = display2.createFrame(); 
        c.registerFrame(displayFrame2);   
        displayFrame2.setVisible(false);  
        display2.setBackdrop(Color.white);
        if(((AgentsSimulation)state).doGraphics){
        	displayFrame.setTitle("Network Diagram");
        	display.attach(edgePortrayal, "Edges");
        	display.attach(nodePortrayal,"Nodes");
        	displayFrame2.setTitle("Network Matrix");
        	display2.attach(ecologyPortrayal, "Ecology");

        }
        ssc.setDisplay(display); //pass to display
           }
    
    
    protected GraphicalUI() { // Constructor for this class
        super(new AgentsSimulation(System.currentTimeMillis()));
        ssc = (SimStateWithSimController)state;
                  // Create a simulation in it
 }
    public GraphicalUI(SimState state) { 
        super(state); // Pass the already created simulation to it
        }

    public static String getName() { 
        return "Agents Simulation"; 
               // return a name for what this simulation is about
         }

    public void quit() {
         super.quit(); // Use the already defined quit method
            
         if (displayFrame!=null) displayFrame.dispose(); 
            displayFrame = null;  // when quiting get rid of the display
            display = null;       
    }
    
    


	public void start() {
          super.start(); // use the predefined start method
           setupPortrayals(); // add setupPortrayals method below

    }

        
    public void load(SimState state) {
        super.load(state); // load the simulation into the interface
        setupPortrayals(); // call setuuPortrayals
    }
    
    public Object getSimulationInspectedObject() {
        return state; // This returns the simulation
   }
    
    public static void main(String[] args) {
    	GraphicalUI ex = new GraphicalUI(); 
        Console c = new Console(ex);
        ssc.setGUIState(ex); //pass to SimState
		c.setVisible(true); // make the console visible
		ssc.setConsole(c); //Pass to Simstate
        System.out.println("Start Simulation"); 
        }
	
}//end class
