package agents;
/*
 * This agent will be stepped last. It will gather all the data needed and will be responsible for
 * writing to any output files. 
 */

import agents.control.SimController;
import sim.engine.Schedule;
import sim.engine.SimState;
import sim.engine.Steppable;
import sim.engine.Stoppable;
import sim.field.continuous.Continuous2D;
import sim.field.grid.SparseGrid2D;
import sim.util.Bag;
import sim.util.IntBag;


public class Observer implements Steppable {
	public final int LARGE_ORDER = Integer.MAX_VALUE; // maximum value of an integer
	public final double INTERVAL = 1.0;
	public HandleData hd = null;
	public HandleData hdAbbrev = null; //this will take just the last line of data.
	public Data data = null;
	public String fileName = "data.txt";
	public String folderName = "data";
	public boolean saveDataToFile = false;
	public boolean savePNGToFile = false;
	public Stoppable stop;
	int[][] test = null;
	public SimController sc = null;
	
	public HandleData hdParameters = null; //use a separate file to record parameters.
	
	public HandleData hdNetwk = null; //this will hold the networks over time.
	public HandleData hdNetwkAbbrev = null; //this will hold just the network at the end of the run.
	
	public HandleData hdGamesPlayed = null;//list of games being played at beginning and end.
	public HandleData hdGamesPlayedAbbr = null;
	
	public HandleData hdAssortment = null; //average assortment for each run. 
	
	public HandleData hdGamePlayerFreq = null; //frequency of cooperators in each game. 
	
	public HandleData hdPlayersCD = null; //histogram data for Cs and Ds in games.
	
	public int count1 = 0;
	public int count2 = 0;
	public long lengthOfSimulations = 1000000; 
	SimState state;
	
	//variables for equilibrium test:
	public double[] payoffs = new double[100];
	public double[] agentsPerGame = new double[100];
	public double[] gamesPerAgent = new double[100];
	public boolean printLongFile = true; //print the time course data?
	public boolean printPlayerFreq = false; //print histogram info for per-game player frequencies
	public boolean printPlayerCD = true;
	
	public Observer(SimState state,Observer ob, String fileName, String folderName, 
			boolean saveDataToFile,boolean savePNGToFile, SimController sc, String precision){
		AgentsSimulation as = (AgentsSimulation)state;

		this.fileName = fileName;
		this.folderName = folderName;
		this.saveDataToFile= saveDataToFile;
		this.savePNGToFile = savePNGToFile;
		this.lengthOfSimulations = as.lengthOfSimulations;
		hd = new HandleData(state, folderName, fileName,saveDataToFile,savePNGToFile,sc,precision);

		data = new Data();
		//parameters file.
		String par = fileName.replace(".txt", "_parameters.txt");
		hdParameters = new HandleData(state, folderName, par, saveDataToFile, savePNGToFile,sc,precision);
		
		//For writing to an abbreviated data file.
		String abbrev = fileName.replace(".txt", "_abbrev.txt");
		hdAbbrev = new HandleData(state, folderName, abbrev, saveDataToFile, savePNGToFile,sc,precision);

		
		//for the network time series.
		String netwk = fileName.replace(".txt", "_net.txt");
		hdNetwk = new HandleData(state, folderName, netwk, saveDataToFile, savePNGToFile,sc,precision);
		
		//for the network time series.
		String netwkAbbr = fileName.replace(".txt", "_netAbbrev.txt");
		hdNetwkAbbrev = new HandleData(state, folderName, netwkAbbr, saveDataToFile, savePNGToFile,sc,precision);
		
		//for the game Info
		String gameInfo = fileName.replace(".txt", "_gameInfo.txt");
		hdGamesPlayed = new HandleData(state, folderName, gameInfo, saveDataToFile, savePNGToFile,sc,precision);
		String gameInfoA = fileName.replace(".txt", "_gameInfoAbbr.txt");
		hdGamesPlayedAbbr = new HandleData(state, folderName, gameInfoA, saveDataToFile, savePNGToFile,sc,precision);
		
		//for assortment
		String assrt = fileName.replace(".txt", "_assort.txt");
		hdAssortment = new HandleData(state, folderName, assrt, saveDataToFile, savePNGToFile,sc,precision);
		
		//for assortment
		String playFreq = fileName.replace(".txt", "_PlayerFreq.txt");
		hdGamePlayerFreq = new HandleData(state, folderName, playFreq, saveDataToFile, savePNGToFile,sc,precision);
		
		//for histogram of players
		String playCD = fileName.replace(".txt", "_PlayerCD.txt");
		hdPlayersCD = new HandleData(state, folderName, playCD, saveDataToFile, savePNGToFile,sc,precision);
		

		this.sc = sc;
	
		if(as.newParameters){
			ob = null;
			as.newParameters = false;
		}

		stop = state.schedule.scheduleRepeating(this,LARGE_ORDER,INTERVAL);
	}//end Constructor method
	
	
	

	public Observer(SimState state){
		stop = state.schedule.scheduleRepeating(this,LARGE_ORDER,INTERVAL);
		// make the observe self scheduling
	}
	/*
	 * We need to synchronize the step for technical reasons since the next step
	 * could start before the observer is done.
	 */
	public synchronized void step (SimState state){
		final AgentsSimulation as = (AgentsSimulation)state;
		final SparseGrid2D agentsSpace = as.agentsSpace;
		final SparseGrid2D gamesSpace = as.gamesSpace;
		final Schedule schedule = state.schedule;
		long thestep = schedule.getSteps();
		final Bag agents = agentsSpace.getAllObjects(); // get all the agents
		final boolean printToConsole = as.printToConsole;
		final boolean printDataRows = as.printDataRows;

		if(as.doGraphics)
			updateEcoGrid(as);
		
		
		//update RCP
		if(as.timePastRCP < 1){
			double RCP = as.returnRCP();
			if(RCP >= 1)
				as.timePastRCP = thestep;
		}
		
		if(thestep < 100){
			payoffs[(int)thestep] = average(as.coopPayoffs);
			agentsPerGame[(int)thestep] = average(as.gameSizesExclusive());
			gamesPerAgent[(int)thestep] = average(as.agentConnectivity());
		}
		else
			updateEquilibriumTest(as);
		
		boolean atEquil = atEquilibrium(as);
		if(atEquil) // stop if empty or if the population is all defectors or all cooperators.
		{
		if(as.autoSimulation)
			printAssortment(as);
		doStop(as);
		stop.stop();
		}
		if(as.autoSimulation && thestep == lengthOfSimulations - 1)
			printAssortment(as);

		// Print initial headings to Console
		if(printToConsole && thestep == 0){ 
			if(sc != null && sc.simulationCount == 1){ //the simcount will have been incremented
				hd.printPramsToConsole(state);
				if(printDataRows)
					hd.printHeadersToConsole(data);
			}
			else {
				hd.printPramsToConsole(state);
				if(printDataRows)
					hd.printHeadersToConsole(data);
			}
		}
		
		//Print initial headings to file. 
		if(saveDataToFile && thestep == 0){ 
			if(sc != null && sc.simulationCount ==1){ //the simcount will have been incremented
				//hdParameters
				String x = "Run: " + sc.runCount;
				hdParameters.write.writeStringln(x);
				hdParameters.printPramsToFile(state);
				if(as.simCount == 1){
					hd.printHeadersToFile2(data);
					hdAbbrev.printHeadersToFile2(data);//print headers to the abbreviated file.
					printAssortmentHeader();
					if(as.printGameInfo)
						printGameInfoHeaders();
					if(printPlayerFreq)
						printPlayerFreqHeaders(as);
					if(printPlayerCD)
						printPlayerCDHeaders(as);
					
				}
			}	
		}
		
		//updatePrintSteps(as, thestep);//not used so far.
		int printSteps = as.printSteps;
		if(thestep > 1000)
			printSteps = 20;
		else if(thestep > 1000)
			printSteps = 10;
		else if(thestep > 400)
			printSteps = 5;
		else 
			printSteps = 1;

		//If the step is a multiple of printSteps or if the run has ended, print data.
		if(thestep%printSteps == 0 || (thestep > 20 && atEquil)){
			if(as.autoSimulation){
				data.runCount = sc.runCount;
				data.simCount = sc.simulationCount;
			}
			else{
				data.runCount = 0;
				data.simCount = as.simCount;	
			}
			data.getData(state);
			if(printDataRows && printToConsole)
				hd.printDataToConsole(data);
			if(printDataRows && saveDataToFile && printLongFile)
				hd.printDataToFile(data);          
			if(saveDataToFile && as.printNetworkData)
				printNetwork(as);
			if(saveDataToFile && printPlayerFreq)
				printPlayerFreq(as);
			
		}
		//if at the end, print the abbreviated data. 
		if(as.autoSimulation && (thestep == lengthOfSimulations - 1 || (thestep > 20 && atEquil))){
			data.runCount = sc.runCount;
			data.simCount = sc.simulationCount;
			data.getData(state);
			hdAbbrev.printDataToFile(data);
			if(as.printNetworkData)
				printNetworkAbbrev(as);
			if(as.printGameInfo){
				as.gameInfoLast();
				printGameInfo(as);
				printGameInfoAbbr(as);
			}
			if(saveDataToFile && printPlayerCD)
				printPlayerCD(as);
		}
			
		
		
	}//end Step Method

	
	/*
	 * Update the ecology grid portrayal
	 */
	public void updateEcoGrid(AgentsSimulation as){
		final boolean[][] gameMatrix = as.gameMatrix;
		
		for(int x = 0; x < as.numAgents; x++)
			for(int y = 0; y < as.numGames; y++){
				if(!gameMatrix[x][y])
					as.ecologyGrid.set(x, y, 0);
			}
		
		Bag agents = as.agentsSpace.getAllObjects();
		int strategy = 0;
		for(int x = 0; x < as.numAgents; x++){
			Agent a = (Agent)agents.get(x);
			strategy = (-1*a.strategy) + 2; //1 for coop, 2 for defector.
			IntBag games = a.currentGamesInd(as);	
			for(int j = 0; j < games.numObjs; j++){
				int y = games.get(j);
				as.ecologyGrid.set(x, y, strategy);
			}	
		}	
	}
	
	
	
	
	
	/**
	 * Method to print the bipartite network to a file.
	 * This one prints just the network at the end of a run.
	 */
	public void printNetworkAbbrev(AgentsSimulation as){
		final boolean[][] gameMatrix = as.gameMatrix;
		final int rows = gameMatrix.length;
		final int col = gameMatrix[0].length;
		String netData = "";;
		for (int i = 0; i < rows; i++){ //go thru each row
			netData = sc.runCount + "\t" + sc.simulationCount + "\t";
			for (int j = 0; j < col; j++){ //go thru each column
				int x = (gameMatrix[i][j]) ? 1 : 0; //1 if True, 0 if False
				netData += x + "\t";
			}
			hdNetwkAbbrev.write.writeStringln(netData);
		}
	}
	
	/**
	 * Method to print the bipartite network to a file.
	 * This one prints the network and the time, for use with the time series.
	 */
	public void printNetwork(AgentsSimulation as){
		final boolean[][] gameMatrix = as.gameMatrix;
		final int rows = gameMatrix.length;
		final int col = gameMatrix[0].length;
		String netData = ""; 
		for (int i = 0; i < rows; i++){ //go thru each row
			netData = sc.runCount + "\t" + sc.simulationCount + "\t" 
									+ as.schedule.getSteps() + "\t";
			for (int j = 0; j < col; j++){ //go thru each column
				int x = (gameMatrix[i][j]) ? 1 : 0; //1 if True, 0 if False
				netData += x + "\t";
			}
			hdNetwk.write.writeStringln(netData);
		}
	}
	
	public void printGameInfoHeaders(){
		//print headers.
		String headers = "run" + "\t" + "sim" + "\t" + "game" + "\t" + "cInit"
		+ "\t" + "dInit" + "\t" + "totalInit" + "\t" + "cFreqInit" + "\t" + "cLast" + 
		"\t" + "dLast" + "\t" + "totalLast" + "\t" + "cFreqLast";
		hdGamesPlayed.write.writeStringln(headers);
		/*
		 * run, sim, % of coopMajorities that survive, %of noncoopMajorities that survive, 
		 * number of new games that with coopMajority, number without.
		 */
		String headers2 = "run" + "\t" + "sim" + "\t" + "totSurvive" + "\t" + 
			"cFreqSurv>=50" + "\t" + "cFreqSurv>50" + "\t" +" totDead" + "\t" + 
			"cFreqDead>=50" + "\t" + "cFreqDead>50" + "\t" + "totNew" + "\t" + 
			"cFreqNew>=50" + "\t" + "cFreqNew>50";
		hdGamesPlayedAbbr.write.writeStringln(headers2);
	}
	
	public void printGameInfo(AgentsSimulation as){
		final double[][] gameInfo = as.gameInfo;
		//print the rest 
		for(int i = 0; i < as.numGames; i++){
			String gameData = "";
			gameData += (sc.runCount+1) + "\t";
			gameData += sc.simulationCount + "\t";
			gameData += i + "\t";
			for(int j = 0; j < gameInfo[0].length; j++)
				gameData += gameInfo[i][j] + "\t";
			hdGamesPlayed.write.writeStringln(gameData);
		}
		
	}
	

	public void printGameInfoAbbr(AgentsSimulation as){
		final double[][] gameInfo = as.gameInfo;
		double totalSurvive = 0;
		double cMajSurvive = 0;
		double cMajSurviveP = 0;//strict majority
		double totalNew = 0;
		double cMajNew = 0;
		double cMajNewP = 0;
		double totalDead = 0;
		double cMajDead = 0;
		double cMajDeadP = 0;
		
		for (int i = 0; i < as.numGames; i++){
			if(gameInfo[i][2] > 0 && gameInfo[i][6] > 0){//survivors
				totalSurvive++;
				if(gameInfo[i][3] >= 0.5)//cFreqInit >= 0.5
					cMajSurvive++;
				if(gameInfo[i][3] > 0.5)//cFreqInit >= 0.5
					cMajSurviveP++;
			}
			else if(gameInfo[i][2] < 1 && gameInfo[i][6] > 0){//new games
				totalNew++;
				if(gameInfo[i][7] >= 0.5)//cFreqLast >= 0.5
					cMajNew++;
				if(gameInfo[i][7] > 0.5)//cFreqLast >= 0.5
					cMajNewP++;
			}
			else if(gameInfo[i][2] > 0 && gameInfo[i][6] < 1){//dead games
				totalDead++;
				if(gameInfo[i][3] >= 0.5)//cFreqInit >= 0.5
					cMajDead++;
				if(gameInfo[i][3] > 0.5)//cFreqInit >= 0.5
					cMajDeadP++;
			}
		}
		String gd = "";
		gd += (sc.runCount+1) + "\t";
		gd += sc.simulationCount + "\t";
		gd += totalSurvive + "\t" + (cMajSurvive/totalSurvive) + "\t" + (cMajSurviveP/totalSurvive) + "\t";
		double cd = (totalDead > 0) ? cMajDead/totalDead : 0;
		double cn = (totalNew > 0) ? cMajNew/totalNew : 0;
		double cdp = (totalDead > 0) ? cMajDeadP/totalDead : 0;
		double cnp = (totalNew > 0) ? cMajNewP/totalNew : 0;
		gd += totalDead + "\t" + cd + "\t" + cdp + "\t";
		gd += totalNew + "\t" + cn + "\t" + cnp;
		
		hdGamesPlayedAbbr.write.writeStringln(gd);
	}
	
	/**
	 * Return true if the system is at equilibrium.
	 * Equilibrium occurs when time > 1 AND either:
	 * 1) all agents are in the same number of games.
	 * 2) All coopPayoffs are equal and all defectorPayoffs are equal.
	 * 3) There is no change to any metrics for X time steps.
	 * 
	 */
	public boolean atEquilibrium(AgentsSimulation as){
		//1. If all agents are in the same number of games. 
	//	if(stdev(as.agentConnectivity()) < .000000001)
	//		return true;
		//2. If all P(c) are equal AND all P(d) are equal.
		if(!as.stopAtEquilibrium)
			return false;
		if(stdev(as.coopPayoffs) < .000000001 && stdev(as.defectPayoffs) < .000000001){
			//System.out.println("Stopped for Reason 2");
			return true;
		}
		//3. If no change.
		else if(as.schedule.getSteps() > 100 && 
				stdev(payoffs) < .000000001 && stdev(agentsPerGame) < .000000001 && stdev(gamesPerAgent) < .000000001){
			//System.out.println("Stopped for Reason 3");
			return true;
		}
		else
			return false;
	}
	
	
	/**
	 * update the equilibrium test files. 
	 */
	public void updateEquilibriumTest(AgentsSimulation as){
		double[] tempPayoffs = new double[100];
		double[] tempAPG = new double[100];
		double[] tempGPA = new double[100];
		for(int i = 0; i < 99; i++){
			tempPayoffs[i] = payoffs[i+1]; 
			tempAPG[i] = agentsPerGame[i+1];
			tempGPA[i] = gamesPerAgent[i+1];
		}
		payoffs = tempPayoffs;
		agentsPerGame = tempAPG;
		gamesPerAgent = tempGPA;
		payoffs[99] = average(as.coopPayoffs);
		agentsPerGame[99] = average(as.gameSizesExclusive());
		gamesPerAgent[99] = average(as.agentConnectivity());
	}
	
	
	/**
	 * returns the mean of an array of doubles
	 */
	public double average(double[] vector){
		double sum = 0;
		final double size = (double)vector.length;
		for(int i = 0; i < size; i++)
			sum += vector[i];
		return sum/size;
	}

	
	/**
     * calculates the standard deviation of an array of double.
     */
    public double stdev(double[] vector){
      double avg = average(vector);
  	  double val = 0.0;
  	  double sumSq = 0.0;
  	  for(int i = 0; i < vector.length; i++)
  		  sumSq += Math.pow((double)(vector[i] - avg), 2.0); //sum of the squares.
  	  val = (1/(double)(vector.length - 1))*sumSq;
  	  val = Math.sqrt(val);
  	  return val;
    }
	
	
	/**
	 * Stop all active agents (end the simulation)
	 */
	public void doStop (SimState state){
		AgentsSimulation as = (AgentsSimulation)state;               
		as.ge.event.stop();
		
		Bag agents = as.agentsSpace.getAllObjects();
		for(int i=0;i<agents.numObjs;i++){
				Agent a = (Agent)agents.get(i);
				a.event.stop();
		}
	}
	
	/**
	 * Prints the assortment index for the run. 
	 */
	public void printAssortment(AgentsSimulation as){
		double assortment = calculateAssortment(as);
		
		String assrt = "";
		assrt += (sc.runCount+1) + "\t";
		assrt += sc.simulationCount + "\t";
		assrt += assortment;
		hdAssortment.write.writeStringln(assrt);
	}
	
	public void printAssortmentHeader(){
		String headers = "run" + "\t" + "sim" + "\t" + "r";
		hdAssortment.write.writeStringln(headers);
	}
	
	
	/** print the number of players of each type in each game - headers*/
	public void printPlayerFreqHeaders(AgentsSimulation as){
		final int numGames = as.numGames;
		String headers = "run" + "\t" + "sim" + "\t" + "time";// + "\t" + "r";
		for(int i = 0; i < numGames; i++){
			headers += "\t" + "C[" + i + "]";
			headers += "\t" + "D[" + i + "]";
		}
		hdGamePlayerFreq.write.writeStringln(headers);
	}
	
	public void printPlayerFreq(AgentsSimulation as){
		String gamePlyrs = "";
		gamePlyrs += (sc.runCount+1) + "\t";
		gamePlyrs += sc.simulationCount + "\t";
		gamePlyrs += as.schedule.getSteps();//time
		final int numGames = as.numGames;
		int c = 0;
		int d = 0;
		Bag games = as.gamesSpace.getAllObjects();
		for(int i = 0; i < numGames; i++){
			Game g = (Game)games.get(i);
			int playrs = g.howManyPlayers(as);
			if(playrs > 0){
				c = g.howManyCooperators(as);
				d = playrs - c;
			}
			else
				{c = 0; d = 0;}
			gamePlyrs += "\t" + c + "\t" + d;
				
		}
		hdGamePlayerFreq.write.writeStringln(gamePlyrs);
	}
	
	
	/** print the number of players of each type in each game - headers*/
	public void printPlayerCDHeaders(AgentsSimulation as){
		final int numAgents = as.numAgents + 1;
		String headers = "run" + "\t" + "sim" + "\t" + "time" + "\t" + "strat";
		for(int i = 0; i < numAgents; i++){
			headers += "\t" + "N[" + i + "]";
		}
		hdPlayersCD.write.writeStringln(headers);
	}
	
	public void printPlayerCD(AgentsSimulation as){
		String gamePlyrsC = "";
		gamePlyrsC += (sc.runCount+1) + "\t";
		gamePlyrsC += sc.simulationCount + "\t";
		gamePlyrsC += as.schedule.getSteps() + "\t";//time
		gamePlyrsC += "C";
		String gamePlyrsD = "";
		gamePlyrsD += (sc.runCount+1) + "\t";
		gamePlyrsD += sc.simulationCount + "\t";
		gamePlyrsD += as.schedule.getSteps() + "\t";//time
		gamePlyrsD += "D";
		
		final double[][] gamesCD = as.gameInfoHist();
		final int numAgents = as.numAgents + 1;
		for(int i = 0; i < numAgents; i++){
			gamePlyrsC += "\t" + gamesCD[0][i];
			gamePlyrsD += "\t" + gamesCD[1][i];	
		}
		hdPlayersCD.write.writeStringln(gamePlyrsC);
		hdPlayersCD.write.writeStringln(gamePlyrsD);
	}
	
	
	/**
	 * Return the average assortment index. 
	 */
	public double calculateAssortment(AgentsSimulation as){
		double assortment = 0;
		Bag agents = as.agentsSpace.getAllObjects();
		for(int i = 0; i < as.numAgents; i++){
			Agent a = (Agent)agents.get(i);
			assortment += a.assortmentIndex(as);
		}
		return assortment/(double)as.numAgents; 
	}

	
	/**
	 * Update the printsteps as time goes on. For the first 
	 */
	public void updatePrintSteps(AgentsSimulation as, long thestep){
		/*
		 * This is so we can update as.printSteps. For example, we may want printSteps to 
		 * be very small early on, and more spread out later (e.g. for a Log-scale).  
		 * 
		 */
		
	}
	
	

	
	
	/**
	 * Return true if should snap a pic
	 */
	public boolean snapPic(AgentsSimulation as, long thestep){
		final int gw = as.gridWidth;
		if(gw == 100)
			return true;
		else
			return false;
	}
	
	
	

}//end of class